/*
 * pkg.c
 * higher-level dependency graph compilation, management and manipulation
 *
 * Copyright (c) 2011 William Pitcock <nenolod@dereferenced.org>.
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include "pkg.h"

pkg_t *
pkg_find(const char *name)
{
	char locbuf[BUFSIZ];

	snprintf(locbuf, sizeof locbuf, "/usr/lib/pkgconfig/%s.pc", name);

	return parse_file(locbuf);
}

void
pkg_traverse(pkg_t *root,
	void (*pkg_traverse_func)(pkg_t *package, void *data),
	void *data,
	int maxdepth)
{
	pkg_dependency_t *node;

	if (maxdepth == 0)
		return;

	foreach_list_entry(root->requires, node)
	{
		pkg_t *pkgdep;

		if (*node->package == '\0')
			continue;

		pkgdep = pkg_find(node->package);
		if (pkgdep == NULL)	
		{
			fprintf(stderr, "Package %s was not found in the pkg-config search path.\n", node->package);
			fprintf(stderr, "Perhaps you should add the directory containing `%s.pc'\n", node->package);
			fprintf(stderr, "to the PKG_CONFIG_PATH environment variable\n");
			fprintf(stderr, "No package '%s' found\n", node->package);
			exit(EXIT_FAILURE);
		}

		pkg_traverse(pkgdep, pkg_traverse_func, data, maxdepth - 1);
	}

	pkg_traverse_func(root, data);
}
